Class: SVMKit::KernelApproximation::RBF

Inherits:
Object
  • Object
show all
Includes:
Base::BaseEstimator, Base::Transformer
Defined in:
lib/svmkit/kernel_approximation/rbf.rb

Overview

Class for RBF kernel feature mapping.

Refernce:

    1. Rahimi and B. Recht, "Random Features for Large-Scale Kernel Machines," Proc. NIPS'07, pp.1177--1184, 2007.

Examples:

transformer = SVMKit::KernelApproximation::RBF.new(gamma: 1.0, n_coponents: 128, random_seed: 1)
new_training_samples = transformer.fit_transform(training_samples)
new_testing_samples = transformer.transform(testing_samples)

Instance Attribute Summary collapse

Attributes included from Base::BaseEstimator

#params

Instance Method Summary collapse

Constructor Details

#new(gamma: 1.0, n_components: 128, random_seed: 1) ⇒ RBF

Create a new transformer for mapping to RBF kernel feature space.

Parameters:

  • params (Hash) (defaults to: {})

    The parameters for RBF kernel approximation.

Options Hash (params):

  • :gamma (Float) — default: 1.0

    The parameter of RBF kernel: exp(-gamma * x^2).

  • :n_components (Integer) — default: 128

    The number of dimensions of the RBF kernel feature space.

  • :random_seed (Integer) — default: nil

    The seed value using to initialize the random generator.



47
48
49
50
51
52
53
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 47

def initialize(params = {})
  self.params = DEFAULT_PARAMS.merge(Hash[params.map { |k, v| [k.to_sym, v] }])
  self.params[:random_seed] ||= srand
  @rng = Random.new(self.params[:random_seed])
  @random_mat = nil
  @random_vec = nil
end

Instance Attribute Details

#random_matNMatrix (readonly)

Return the random matrix for transformation.

Returns:

  • (NMatrix)

    (shape: [n_features, n_components])



29
30
31
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 29

def random_mat
  @random_mat
end

#random_vecNMatrix (readonly)

Return the random vector for transformation.

Returns:

  • (NMatrix)

    (shape: [1, n_components])



33
34
35
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 33

def random_vec
  @random_vec
end

#rngRandom (readonly)

Return the random generator for transformation.

Returns:

  • (Random)


37
38
39
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 37

def rng
  @rng
end

Instance Method Details

#fit(x) ⇒ RBF

Fit the model with given training data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The training data to be used for fitting the model. This method uses only the number of features of the data.

Returns:

  • (RBF)

    The learned transformer itself.



62
63
64
65
66
67
68
69
70
71
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 62

def fit(x, _y = nil)
  n_features = x.shape[1]
  params[:n_components] = 2 * n_features if params[:n_components] <= 0
  @random_mat = rand_normal([n_features, params[:n_components]]) * (2.0 * params[:gamma])**0.5
  n_half_components = params[:n_components] / 2
  @random_vec = NMatrix.zeros([1, params[:n_components] - n_half_components]).hconcat(
    NMatrix.ones([1, n_half_components]) * (0.5 * Math::PI)
  )
  self
end

#fit_transform(x) ⇒ NMatrix

Fit the model with training data, and then transform them with the learned model.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The training data to be used for fitting the model.

Returns:

  • (NMatrix)

    (shape: [n_samples, n_components]) The transformed data



79
80
81
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 79

def fit_transform(x, _y = nil)
  fit(x).transform(x)
end

#marshal_dumpHash

Dump marshal data.

Returns:

  • (Hash)

    The marshal data about RBF.



97
98
99
100
101
102
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 97

def marshal_dump
  { params: params,
    random_mat: Utils.dump_nmatrix(@random_mat),
    random_vec: Utils.dump_nmatrix(@random_vec),
    rng: @rng }
end

#marshal_load(obj) ⇒ nil

Load marshal data.

Returns:

  • (nil)


106
107
108
109
110
111
112
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 106

def marshal_load(obj)
  self.params = obj[:params]
  @random_mat = Utils.restore_nmatrix(obj[:random_mat])
  @random_vec = Utils.restore_nmatrix(obj[:random_vec])
  @rng = obj[:rng]
  nil
end

#transform(x) ⇒ NMatrix

Transform the given data with the learned model.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The data to be transformed with the learned model.

Returns:

  • (NMatrix)

    (shape: [n_samples, n_components]) The transformed data.



89
90
91
92
93
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 89

def transform(x)
  n_samples, = x.shape
  projection = x.dot(@random_mat) + @random_vec.repeat(n_samples, 0)
  projection.sin * ((2.0 / params[:n_components])**0.5)
end