Class: SVMKit::LinearModel::PegasosSVC

Inherits:
Object
  • Object
show all
Includes:
Base::BaseEstimator, Base::Classifier
Defined in:
lib/svmkit/linear_model/pegasos_svc.rb

Overview

PegasosSVC is a class that implements Support Vector Classifier with the Pegasos algorithm.

Reference

    1. Shalev-Shwartz and Y. Singer, "Pegasos: Primal Estimated sub-GrAdient SOlver for SVM," Proc. ICML'07, pp. 807--814, 2007.

Examples:

estimator =
  SVMKit::LinearModel::PegasosSVC.new(reg_param: 1.0, max_iter: 100, batch_size: 20, random_seed: 1)
estimator.fit(training_samples, traininig_labels)
results = estimator.predict(testing_samples)

Instance Attribute Summary collapse

Attributes included from Base::BaseEstimator

#params

Instance Method Summary collapse

Constructor Details

#new(reg_param: 1.0, max_iter: 100, batch_size: 50, random_seed: 1) ⇒ PegasosSVC

Create a new classifier with Support Vector Machine by the Pegasos algorithm.

Parameters:

  • params (Hash) (defaults to: {})

    The parameters for SVC.

Options Hash (params):

  • :reg_param (Float) — default: 1.0

    The regularization parameter.

  • :fit_bias (Boolean) — default: false

    The flag indicating whether to fit the bias term.

  • :bias_scale (Float) — default: 1.0

    The scale of the bias term.

  • :max_iter (Integer) — default: 100

    The maximum number of iterations.

  • :batch_size (Integer) — default: 50

    The size of the mini batches.

  • :random_seed (Integer) — default: nil

    The seed value using to initialize the random generator.



54
55
56
57
58
59
60
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 54

def initialize(params = {})
  self.params = DEFAULT_PARAMS.merge(Hash[params.map { |k, v| [k.to_sym, v] }])
  self.params[:random_seed] ||= srand
  @weight_vec = nil
  @bias_term = 0.0
  @rng = Random.new(self.params[:random_seed])
end

Instance Attribute Details

#bias_termFloat (readonly)

Return the bias term (a.k.a. intercept) for SVC.

Returns:

  • (Float)


37
38
39
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 37

def bias_term
  @bias_term
end

#rngRandom (readonly)

Return the random generator for performing random sampling in the Pegasos algorithm.

Returns:

  • (Random)


41
42
43
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 41

def rng
  @rng
end

#weight_vecNMatrix (readonly)

Return the weight vector for SVC.

Returns:

  • (NMatrix)

    (shape: [1, n_features])



33
34
35
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 33

def weight_vec
  @weight_vec
end

Instance Method Details

#decision_function(x) ⇒ NMatrix

Calculate confidence scores for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to compute the scores.

Returns:

  • (NMatrix)

    (shape: [1, n_samples]) Confidence score per sample.



113
114
115
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 113

def decision_function(x)
  @weight_vec.dot(x.transpose) + @bias_term
end

#fit(x, y) ⇒ PegasosSVC

Fit the model with given training data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The training data to be used for fitting the model.

  • y (NMatrix)

    (shape: [1, n_samples]) The labels to be used for fitting the model.

Returns:



67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 67

def fit(x, y)
  # Generate binary labels
  negative_label = y.uniq.sort.shift
  bin_y = y.to_flat_a.map { |l| l != negative_label ? 1 : -1 }
  # Expand feature vectors for bias term.
  samples = x
  samples = samples.hconcat(NMatrix.ones([x.shape[0], 1]) * params[:bias_scale]) if params[:fit_bias]
  # Initialize some variables.
  n_samples, n_features = samples.shape
  rand_ids = [*0..n_samples - 1].shuffle(random: @rng)
  weight_vec = NMatrix.zeros([1, n_features])
  # Start optimization.
  params[:max_iter].times do |t|
    # random sampling
    subset_ids = rand_ids.shift(params[:batch_size])
    rand_ids.concat(subset_ids)
    target_ids = subset_ids.map do |n|
      n if weight_vec.dot(samples.row(n).transpose) * bin_y[n] < 1
    end
    n_subsamples = target_ids.size
    next if n_subsamples.zero?
    # update the weight vector.
    eta = 1.0 / (params[:reg_param] * (t + 1))
    mean_vec = NMatrix.zeros([1, n_features])
    target_ids.each { |n| mean_vec += samples.row(n) * bin_y[n] }
    mean_vec *= eta / n_subsamples
    weight_vec = weight_vec * (1.0 - eta * params[:reg_param]) + mean_vec
    # scale the weight vector.
    scaler = (1.0 / params[:reg_param]**0.5) / weight_vec.norm2
    weight_vec *= [1.0, scaler].min
  end
  # Store the learned model.
  if params[:fit_bias]
    @weight_vec = weight_vec[0...n_features - 1]
    @bias_term = weight_vec[n_features - 1]
  else
    @weight_vec = weight_vec[0...n_features]
    @bias_term = 0.0
  end
  self
end

#marshal_dumpHash

Dump marshal data.

Returns:

  • (Hash)

    The marshal data about PegasosSVC.



138
139
140
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 138

def marshal_dump
  { params: params, weight_vec: Utils.dump_nmatrix(@weight_vec), bias_term: @bias_term, rng: @rng }
end

#marshal_load(obj) ⇒ nil

Load marshal data.

Returns:

  • (nil)


144
145
146
147
148
149
150
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 144

def marshal_load(obj)
  self.params = obj[:params]
  @weight_vec = Utils.restore_nmatrix(obj[:weight_vec])
  @bias_term = obj[:bias_term]
  @rng = obj[:rng]
  nil
end

#predict(x) ⇒ NMatrix

Predict class labels for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to predict the labels.

Returns:

  • (NMatrix)

    (shape: [1, n_samples]) Predicted class label per sample.



121
122
123
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 121

def predict(x)
  decision_function(x).map { |v| v >= 0 ? 1 : -1 }
end

#score(x, y) ⇒ Float

Claculate the mean accuracy of the given testing data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) Testing data.

  • y (NMatrix)

    (shape: [1, n_samples]) True labels for testing data.

Returns:

  • (Float)

    Mean accuracy



130
131
132
133
134
# File 'lib/svmkit/linear_model/pegasos_svc.rb', line 130

def score(x, y)
  p = predict(x)
  n_hits = (y.to_flat_a.map.with_index { |l, n| l == p[n] ? 1 : 0 }).inject(:+)
  n_hits / y.size.to_f
end