Class: SVMKit::Multiclass::OneVsRestClassifier

Inherits:
Object
  • Object
show all
Includes:
Base::BaseEstimator, Base::Classifier
Defined in:
lib/svmkit/multiclass/one_vs_rest_classifier.rb

Overview

OneVsRestClassifier is a class that implements One-vs-Rest (OvR) strategy for multi-label classification.

Examples:

base_estimator =
 SVMKit::LinearModel::PegasosSVC.new(penalty: 1.0, max_iter: 100, batch_size: 20, random_seed: 1)
estimator = SVMKit::Multiclass::OneVsRestClassifier.new(estimator: base_estimator)
estimator.fit(training_samples, training_labels)
results = estimator.predict(testing_samples)

Instance Attribute Summary collapse

Attributes included from Base::BaseEstimator

#params

Instance Method Summary collapse

Constructor Details

#new(estimator: base_estimator) ⇒ OneVsRestClassifier

Create a new multi-label classifier with the one-vs-rest startegy.

Parameters:

  • params (Hash) (defaults to: {})

    The parameters for OneVsRestClassifier.

Options Hash (params):

  • :estimator (Classifier) — default: nil

    The (binary) classifier for construction a multi-label classifier.



38
39
40
41
42
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 38

def initialize(params = {})
  self.params = DEFAULT_PARAMS.merge(Hash[params.map { |k, v| [k.to_sym, v] }])
  @estimators = nil
  @classes = nil
end

Instance Attribute Details

#classesNMatrix (readonly)

Return the class labels.

Returns:

  • (NMatrix)

    (shape: [1, n_classes])



30
31
32
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 30

def classes
  @classes
end

#estimatorsArray<Classifier> (readonly)

Return the set of estimators.

Returns:

  • (Array<Classifier>)


26
27
28
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 26

def estimators
  @estimators
end

Instance Method Details

#decision_function(x) ⇒ NMatrix

Calculate confidence scores for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to compute the scores.

Returns:

  • (NMatrix)

    (shape: [n_samples, n_classes]) Confidence scores per sample for each class.



62
63
64
65
66
67
68
69
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 62

def decision_function(x)
  n_samples, = x.shape
  n_classes = @classes.size
  NMatrix.new(
    [n_classes, n_samples],
    Array.new(n_classes) { |m| @estimators[m].decision_function(x).to_a }.flatten
  ).transpose
end

#fit(x, y) ⇒ OneVsRestClassifier

Fit the model with given training data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The training data to be used for fitting the model.

  • y (NMatrix)

    (shape: [1, n_samples]) The labels to be used for fitting the model.

Returns:



49
50
51
52
53
54
55
56
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 49

def fit(x, y)
  @classes = y.uniq.sort
  @estimators = @classes.map do |label|
    bin_y = y.map { |l| l == label ? 1 : -1 }
    params[:estimator].dup.fit(x, bin_y)
  end
  self
end

#marshal_dumpHash

Dump marshal data.

Returns:

  • (Hash)

    The marshal data about OneVsRestClassifier.



95
96
97
98
99
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 95

def marshal_dump
  { params: params,
    classes: @classes,
    estimators: @estimators.map { |e| Marshal.dump(e) } }
end

#marshal_load(obj) ⇒ nil

Load marshal data.

Returns:

  • (nil)


103
104
105
106
107
108
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 103

def marshal_load(obj)
  self.params = obj[:params]
  @classes = obj[:classes]
  @estimators = obj[:estimators].map { |e| Marshal.load(e) }
  nil
end

#predict(x) ⇒ NMatrix

Predict class labels for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to predict the labels.

Returns:

  • (NMatrix)

    (shape: [1, n_samples]) Predicted class label per sample.



75
76
77
78
79
80
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 75

def predict(x)
  n_samples, = x.shape
  decision_values = decision_function(x)
  NMatrix.new([1, n_samples],
              decision_values.each_row.map { |vals| @classes[vals.to_a.index(vals.to_a.max)] })
end

#score(x, y) ⇒ Float

Claculate the mean accuracy of the given testing data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) Testing data.

  • y (NMatrix)

    (shape: [1, n_samples]) True labels for testing data.

Returns:

  • (Float)

    Mean accuracy



87
88
89
90
91
# File 'lib/svmkit/multiclass/one_vs_rest_classifier.rb', line 87

def score(x, y)
  p = predict(x)
  n_hits = (y.to_flat_a.map.with_index { |l, n| l == p[n] ? 1 : 0 }).inject(:+)
  n_hits / y.size.to_f
end