Class: SVMKit::Preprocessing::StandardScaler
- Inherits:
-
Object
- Object
- SVMKit::Preprocessing::StandardScaler
- Includes:
- Base::BaseEstimator, Base::Transformer
- Defined in:
- lib/svmkit/preprocessing/standard_scaler.rb
Overview
Normalize samples by centering and scaling to unit variance.
Instance Attribute Summary collapse
-
#mean_vec ⇒ NMatrix
readonly
Return the vector consists of the mean value for each feature.
-
#std_vec ⇒ NMatrix
readonly
Return the vector consists of the standard deviation for each feature.
Attributes included from Base::BaseEstimator
Instance Method Summary collapse
-
#fit(x) ⇒ StandardScaler
Calculate the mean value and standard deviation of each feature for scaling.
-
#fit_transform(x) ⇒ NMatrix
Calculate the mean values and standard deviations, and then normalize samples using them.
-
#new ⇒ StandardScaler
constructor
Create a new normalizer for centering and scaling to unit variance.
-
#marshal_dump ⇒ Hash
Dump marshal data.
-
#marshal_load(obj) ⇒ nil
Load marshal data.
-
#transform(x) ⇒ NMatrix
Perform standardization the given samples.
Constructor Details
#new ⇒ StandardScaler
Create a new normalizer for centering and scaling to unit variance.
28 29 30 31 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 28 def initialize(_params = {}) @mean_vec = nil @std_vec = nil end |
Instance Attribute Details
#mean_vec ⇒ NMatrix (readonly)
Return the vector consists of the mean value for each feature.
19 20 21 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 19 def mean_vec @mean_vec end |
#std_vec ⇒ NMatrix (readonly)
Return the vector consists of the standard deviation for each feature.
23 24 25 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 23 def std_vec @std_vec end |
Instance Method Details
#fit(x) ⇒ StandardScaler
Calculate the mean value and standard deviation of each feature for scaling.
40 41 42 43 44 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 40 def fit(x, _y = nil) @mean_vec = x.mean(0) @std_vec = x.std(0) self end |
#fit_transform(x) ⇒ NMatrix
Calculate the mean values and standard deviations, and then normalize samples using them.
53 54 55 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 53 def fit_transform(x, _y = nil) fit(x).transform(x) end |
#marshal_dump ⇒ Hash
Dump marshal data.
68 69 70 71 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 68 def marshal_dump { mean_vec: Utils.dump_nmatrix(@mean_vec), std_vec: Utils.dump_nmatrix(@std_vec) } end |
#marshal_load(obj) ⇒ nil
Load marshal data.
75 76 77 78 79 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 75 def marshal_load(obj) @mean_vec = Utils.restore_nmatrix(obj[:mean_vec]) @std_vec = Utils.restore_nmatrix(obj[:std_vec]) nil end |
#transform(x) ⇒ NMatrix
Perform standardization the given samples.
61 62 63 64 |
# File 'lib/svmkit/preprocessing/standard_scaler.rb', line 61 def transform(x) n_samples, = x.shape (x - @mean_vec.repeat(n_samples, 0)) / @std_vec.repeat(n_samples, 0) end |