Class: SVMKit::KernelApproximation::RBF
- Inherits:
-
Object
- Object
- SVMKit::KernelApproximation::RBF
- Includes:
- Base::BaseEstimator, Base::Transformer
- Defined in:
- lib/svmkit/kernel_approximation/rbf.rb
Overview
Class for RBF kernel feature mapping.
Refernce:
-
- Rahimi and B. Recht, "Random Features for Large-Scale Kernel Machines," Proc. NIPS'07, pp.1177--1184, 2007.
Instance Attribute Summary collapse
-
#random_mat ⇒ NMatrix
readonly
Return the random matrix for transformation.
-
#random_vec ⇒ NMatrix
readonly
Return the random vector for transformation.
-
#rng ⇒ Random
readonly
Return the random generator for transformation.
Attributes included from Base::BaseEstimator
Instance Method Summary collapse
-
#fit(x) ⇒ RBF
Fit the model with given training data.
-
#fit_transform(x) ⇒ NMatrix
Fit the model with training data, and then transform them with the learned model.
-
#new(gamma: 1.0, n_components: 128, random_seed: 1) ⇒ RBF
constructor
Create a new transformer for mapping to RBF kernel feature space.
-
#marshal_dump ⇒ Hash
Dump marshal data.
-
#marshal_load(obj) ⇒ nil
Load marshal data.
-
#transform(x) ⇒ NMatrix
Transform the given data with the learned model.
Constructor Details
#new(gamma: 1.0, n_components: 128, random_seed: 1) ⇒ RBF
Create a new transformer for mapping to RBF kernel feature space.
46 47 48 49 50 51 52 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 46 def initialize(params = {}) self.params = DEFAULT_PARAMS.merge(Hash[params.map { |k, v| [k.to_sym, v] }]) self.params[:random_seed] ||= srand @rng = Random.new(self.params[:random_seed]) @random_mat = nil @random_vec = nil end |
Instance Attribute Details
#random_mat ⇒ NMatrix (readonly)
Return the random matrix for transformation.
29 30 31 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 29 def random_mat @random_mat end |
#random_vec ⇒ NMatrix (readonly)
Return the random vector for transformation.
33 34 35 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 33 def random_vec @random_vec end |
#rng ⇒ Random (readonly)
Return the random generator for transformation.
37 38 39 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 37 def rng @rng end |
Instance Method Details
#fit(x) ⇒ RBF
Fit the model with given training data.
61 62 63 64 65 66 67 68 69 70 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 61 def fit(x, _y = nil) n_features = x.shape[1] params[:n_components] = 2 * n_features if params[:n_components] <= 0 @random_mat = rand_normal([n_features, params[:n_components]]) * (2.0 * params[:gamma])**0.5 n_half_components = params[:n_components] / 2 @random_vec = NMatrix.zeros([1, params[:n_components] - n_half_components]).hconcat( NMatrix.ones([1, n_half_components]) * (0.5 * Math::PI) ) self end |
#fit_transform(x) ⇒ NMatrix
Fit the model with training data, and then transform them with the learned model.
78 79 80 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 78 def fit_transform(x, _y = nil) fit(x).transform(x) end |
#marshal_dump ⇒ Hash
Dump marshal data.
96 97 98 99 100 101 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 96 def marshal_dump { params: params, random_mat: Utils.dump_nmatrix(@random_mat), random_vec: Utils.dump_nmatrix(@random_vec), rng: @rng } end |
#marshal_load(obj) ⇒ nil
Load marshal data.
105 106 107 108 109 110 111 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 105 def marshal_load(obj) self.params = obj[:params] @random_mat = Utils.restore_nmatrix(obj[:random_mat]) @random_vec = Utils.restore_nmatrix(obj[:random_vec]) @rng = obj[:rng] nil end |
#transform(x) ⇒ NMatrix
Transform the given data with the learned model.
88 89 90 91 92 |
# File 'lib/svmkit/kernel_approximation/rbf.rb', line 88 def transform(x) n_samples, = x.shape projection = x.dot(@random_mat) + @random_vec.repeat(n_samples, 0) projection.sin * ((2.0 / params[:n_components])**0.5) end |