Class: SVMKit::LinearModel::LogisticRegression

Inherits:
Object
  • Object
show all
Includes:
Base::BaseEstimator, Base::Classifier
Defined in:
lib/svmkit/linear_model/logistic_regression.rb

Overview

LogisticRegression is a class that implements Logistic Regression with stochastic gradient descent (SGD) optimization. Note that the class performs as a binary classifier.

Reference

    1. Shalev-Shwartz, Y. Singer, N. Srebro, and A. Cotter, "Pegasos: Primal Estimated sub-GrAdient SOlver for SVM," Mathematical Programming, vol. 127 (1), pp. 3--30, 2011.

Examples:

estimator =
  SVMKit::LinearModel::LogisticRegression.new(reg_param: 1.0, max_iter: 100, batch_size: 20, random_seed: 1)
estimator.fit(training_samples, traininig_labels)
results = estimator.predict(testing_samples)

Instance Attribute Summary collapse

Attributes included from Base::BaseEstimator

#params

Instance Method Summary collapse

Constructor Details

#new(reg_param: 1.0, max_iter: 100, batch_size: 50, random_seed: 1) ⇒ LogisticRegression

Create a new classifier with Logisitc Regression by the SGD optimization.

Parameters:

  • reg_param (Float)

    (defaults to: 1.0) The regularization parameter.

  • fit_bias (Boolean)

    (defaults to: false) The flag indicating whether to fit the bias term.

  • bias_scale (Float)

    (defaults to: 1.0) The scale of the bias term. If fit_bias is true, the feature vector v becoms [v; bias_scale].

  • max_iter (Integer)

    (defaults to: 100) The maximum number of iterations.

  • batch_size (Integer)

    (defaults to: 50) The size of the mini batches.

  • random_seed (Integer)

    (defaults to: nil) The seed value using to initialize the random generator.



56
57
58
59
60
61
62
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 56

def initialize(params = {})
  self.params = DEFAULT_PARAMS.merge(Hash[params.map { |k, v| [k.to_sym, v] }])
  self.params[:random_seed] ||= srand
  @weight_vec = nil
  @bias_term = 0.0
  @rng = Random.new(self.params[:random_seed])
end

Instance Attribute Details

#bias_termFloat (readonly)

Return the bias term (a.k.a. intercept) for Logistic Regression.

Returns:

  • (Float)


39
40
41
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 39

def bias_term
  @bias_term
end

#rngRandom (readonly)

Return the random generator for transformation.

Returns:

  • (Random)


43
44
45
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 43

def rng
  @rng
end

#weight_vecNMatrix (readonly)

Return the weight vector for Logistic Regression.

Returns:

  • (NMatrix)

    (shape: [1, n_features])



35
36
37
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 35

def weight_vec
  @weight_vec
end

Instance Method Details

#decision_function(x) ⇒ NMatrix

Calculate confidence scores for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to compute the scores.

Returns:

  • (NMatrix)

    (shape: [1, n_samples]) Confidence score per sample.



115
116
117
118
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 115

def decision_function(x)
  w = ((@weight_vec.dot(x.transpose) + @bias_term) * -1.0).exp + 1.0
  w.map { |v| 1.0 / v }
end

#fit(x, y) ⇒ LogisticRegression

Fit the model with given training data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The training data to be used for fitting the model.

  • y (NMatrix)

    (shape: [1, n_samples]) The categorical variables (e.g. labels) to be used for fitting the model.

Returns:



70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 70

def fit(x, y)
  # Generate binary labels.
  negative_label = y.uniq.sort.shift
  bin_y = y.to_flat_a.map { |l| l != negative_label ? 1 : 0 }
  # Expand feature vectors for bias term.
  samples = x
  samples = samples.hconcat(NMatrix.ones([x.shape[0], 1]) * params[:bias_scale]) if params[:fit_bias]
  # Initialize some variables.
  n_samples, n_features = samples.shape
  rand_ids = [*0..n_samples - 1].shuffle(random: @rng)
  weight_vec = NMatrix.zeros([1, n_features])
  # Start optimization.
  params[:max_iter].times do |t|
    # random sampling
    subset_ids = rand_ids.shift(params[:batch_size])
    rand_ids.concat(subset_ids)
    # update the weight vector.
    eta = 1.0 / (params[:reg_param] * (t + 1))
    mean_vec = NMatrix.zeros([1, n_features])
    subset_ids.each do |n|
      z = weight_vec.dot(samples.row(n).transpose)[0]
      coef = bin_y[n] / (1.0 + Math.exp(bin_y[n] * z))
      mean_vec += samples.row(n) * coef
    end
    mean_vec *= eta / params[:batch_size]
    weight_vec = weight_vec * (1.0 - eta * params[:reg_param]) + mean_vec
    # scale the weight vector.
    scaler = (1.0 / params[:reg_param]**0.5) / weight_vec.norm2
    weight_vec *= [1.0, scaler].min
  end
  # Store the learned model.
  if params[:fit_bias]
    @weight_vec = weight_vec[0...n_features - 1]
    @bias_term = weight_vec[n_features - 1]
  else
    @weight_vec = weight_vec[0...n_features]
    @bias_term = 0.0
  end
  self
end

#marshal_dumpHash

Dump marshal data.

Returns:

  • (Hash)

    The marshal data about LogisticRegression.



149
150
151
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 149

def marshal_dump
  { params: params, weight_vec: Utils.dump_nmatrix(@weight_vec), bias_term: @bias_term, rng: @rng }
end

#marshal_load(obj) ⇒ nil

Load marshal data.

Returns:

  • (nil)


155
156
157
158
159
160
161
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 155

def marshal_load(obj)
  self.params = obj[:params]
  @weight_vec = Utils.restore_nmatrix(obj[:weight_vec])
  @bias_term = obj[:bias_term]
  @rng = obj[:rng]
  nil
end

#predict(x) ⇒ NMatrix

Predict class labels for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to predict the labels.

Returns:

  • (NMatrix)

    (shape: [1, n_samples]) Predicted class label per sample.



124
125
126
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 124

def predict(x)
  decision_function(x).map { |v| v >= 0.5 ? 1 : -1 }
end

#predict_proba(x) ⇒ NMatrix

Predict probability for samples.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) The samples to predict the probailities.

Returns:

  • (NMatrix)

    (shape: [1, n_samples]) Predicted probability per sample.



132
133
134
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 132

def predict_proba(x)
  decision_function(x)
end

#score(x, y) ⇒ Float

Claculate the mean accuracy of the given testing data.

Parameters:

  • x (NMatrix)

    (shape: [n_samples, n_features]) Testing data.

  • y (NMatrix)

    (shape: [1, n_samples]) True labels for testing data.

Returns:

  • (Float)

    Mean accuracy



141
142
143
144
145
# File 'lib/svmkit/linear_model/logistic_regression.rb', line 141

def score(x, y)
  p = predict(x)
  n_hits = (y.to_flat_a.map.with_index { |l, n| l == p[n] ? 1 : 0 }).inject(:+)
  n_hits / y.size.to_f
end